@@ -5,15 +5,15 @@ extern SQLRETURN sqlFreeHandleEnv(SQLHANDLE hdl);
 
 typedef struct TAG_finalizeonce {
   void *encapobj;
+  int refcount;
   int isfinalized;
   void *extrainfo;
+  struct TAG_finalizeonce *parent;
 } finalizeonce;
 
-extern finalizeonce *wrapobj(void *obj);
-extern finalizeonce *wrapobj_extra(void *obj, void *extra);
-
-extern void sqlFreeHandleEnv_app(finalizeonce *res);
-extern void sqlFreeHandleEnv_finalizer(finalizeonce *res);
+extern finalizeonce *wrapobj(void *obj, finalizeonce *parentobj);
+extern finalizeonce *wrapobj_extra(void *obj, void *extra,
+                                   finalizeonce *parentobj);
 
 extern SQLRETURN sqlFreeHandleDbc_app(finalizeonce *res);
 extern void sqlFreeHandleDbc_finalizer(finalizeonce *res);
@@ -26,3 +26,6 @@ extern void *getSqlOvOdbc3(void);
 
 extern SQLRETURN disableAutoCommit(SQLHDBC conn);
 extern SQLRETURN simpleSqlTables(SQLHSTMT stmt);
+extern SQLRETURN simpleSqlColumns(SQLHSTMT stmt, SQLCHAR *tablename, 
+                                  SQLSMALLINT tnlen);
+
